//            Контроллер лампового усилителя мощности
//        измеритель мощности и КСВ, тока и напряжения анода
//                тока и напряжения второй сетки
//                      UT0IS/D0ISM, 01/2020
//
//  v.1.0.2 - сделал прямой доступ к аналоговым портам
//  v.1.0.3 - подключение кнопки S1 - ON/OFF
//  длинное нажатие S1 - включение, повторное нажатие - выключение
//  введен алгоритм подачи напряжения при включении
//  введен алгоритм снятия напряжений с ламп(ы) при выключении
//  применено два раздельных счетчика на нагрев и охлаждение
//  v.1.0.4 - подключил датчик температуры DS18B20
//  при коротком нажатии кнопки S1 в нижней строке отображается температура
//  и время от мемента включения усилителя
//  повторное нажатие кнопки возвращает индикацию
//  убрал баг в отображении остаточных значений от преобразования АЦП
//  улучшил алгоритм отрисовки градусника, должен работать чуть быстрее
//  v.1.0.5 - подключил функцию гашения подсветки индикатора, если в
//  течении некоторого времени отсутствовал сигнал прямой волны
//  Если сигнал появляется, подсветка автоматически включается
//  При более длительном отсутствии сигнала прямой волны контроллер
//  автоматически переводит усилитель в Bypass, соблюдая всю последовательность
//  отключения напряжений
//  v.1.0.6 - введена защита от высокого КСВ, порог срабатывания регулируется
//  параметром SWR_HI, при этом отключаются напряжение экранной сетки и 
//  анодного напряжения. Короткое нажатие кнопки S1 возвращает рабочий режим
//  усилителя, включаются анодное и экранное напряжения.
//  Каждое короткое нажатие обнуляет таймер режима сна TIME_SLEEP.
//  v.1.0.7 - введен для удобства обратный таймер для режимов нагрева и охлаждения
//  Надпись высокого КСВ HI переделана на HIGH
//  Введена защита блокировки экранного напряжения при просадке/исчезновения
//  анодного напряжения как при нагреве, так и в работе усилителя
//  Изменена защита от высокого КСВ, отключается только экранное напряжение
//  Сброс защиты происходит по короткому нажатию кнопки S1, при этом 
//  подключается экранное напряжение
//  v.1.0.8 - введено автоматическое определение диапазона по частоте
//  излучаемого сигнала, при этом формируются сигналы для дешифратора
//  диапазонов. В память сохраняется последний рабочий диапазон и частота.
//  При включении контроллера загружается последний рабочий диапазон.
//  Введен цифровой VOX, при наличии сигнала высокой частоты
//  формируется сигнал PTT для перехода усилителя на передачу. Сигнал PTT
//  снимается с задержкой VOXDelay, которую можно подобрать индивидуально.
//  Внедрена задержка DELAY_SWITCH сигнала PTT, если новый диапазон отличается от
//  предыдущего для переключения контактов реле. Переделан бипер и сигнал аварии.
//  Продублирована защита от высокого КСВ и пропадания/снижения анодного напряжения.
//  При этом блокируется сигнал передачи PTT. 

#include "GyverTimer.h"                                                             // библиотека таймера
GTimer beep   (MS);                                                                 // таймер бипера
GTimer error  (MS);                                                                 // таймер сигнала аварии
#include <EEPROM.h>                                                                 // библиотека EEPROM
#include <microWire.h>                                                              // библиотека microWire.h
#include <microLiquidCrystal_I2C.h>                                                 // библиотека microLiquidCrystal_I2C.h
#include <sav_button.h>                                                             // подключаем кнопки
SButton button1(1,20,500,0,0);                                                      // кнопка 1 порт D1, короткое нажатие 20мс, длинное нажатие 500мс
LiquidCrystal_I2C lcd(0x27,20,4);                                                   // указываем адрес I2C и размерность дисплея
//#include "GyverFilters.h"
//GFilterRA filter;
#include "microDS18B20.h"                                                           // подключаем библиотеку датчика температуры DS18B20
MicroDS18B20 sensor(6);                                                             // датчик подключен к порту D6

//#define PWRMax          1000                                                        // максимальная выходная мощность (резерв)
//#define PWRStep         20                                                          // округление мощности
//#define U_ANOD_MAX      3200                                                        // максимальное значение анодного напряжения
//#define U_ANOD_ROUND    10                                                          // округляем показания напряжения анода
//#define I_ANOD_MAX      999                                                         // максимальное значение анодного тока (мА)
//#define I_ANOD_ROUND    1                                                           // округляем показания тока анода
//#define U_GRID2_MAX     650                                                         // максимальное значение напряжения второй сетки
//#define I_GRID2_MAX     250                                                         // максимальный ток второй сетки (мА)
//#define U_GRID2_ROUND   10                                                          // округляем показания напряжения второй сетки
//#define TIME_ON         40                                                          // время включения (нагрева), сек
//#define TIME_OFF        60                                                          // время выключения (охлаждения), сек
//#define TIME_LIGHT      2                                                           // задержка на отключение подсветки ЖКИ (в минутах)
//#define TIME_SLEEP      10                                                          // задержка на переход в спящий режим (в минутах)
//#define STEP            10                                                          // шаг уменьшения градусника
//#define UPDATE_TIME     50                                                          // время удержания градусника
//#define SWR_HI          5                                                           // значение КСВ, выше которого блокируется работа усилителя
//#define VOXDelay        10                                                          // задержка отпускания VOX
//#define DELAY_SWITCH    50                                                          // задержка на переключение реле диапазонов, мс

#define TX              2                                                           // номер порта сигнала PTT
#define D0              3                                                           // выход порта дешифратора D0
#define D1              4                                                           // выход порта дешифратора D1
#define FREQ            5                                                           // номер порта входа частотомера
#define D2              7                                                           // выход порта дешифратора D2
#define D3              8                                                           // выход порта дешифратора D3
#define ANOD            9                                                           // порт D9 вкл./откл. напряжения анода
#define FAN             10                                                          // порт D10 вкл./откл. вентилятора охлаждения
#define GLOW            11                                                          // порт D11 вкл./откл. накала ламп
#define SCREEN          12                                                          // порт D12 вкл./откл. напряжения экр. сетки
#define SOUND           13                                                          // номер порта бипера сигнализации высокого КСВ

#define dotimes(n,code) for (int i=0; i<(n); ++i) code;                             // цикл вывода текста заставки
#define text1 " Power Control Unit"                                                 // 1-я строка заставки
#define text2 "  SWR & PWR-meter  "                                                 // 2-я строка заставки
#define text3 "    UT0IS/D0ISM    "                                                 // 3-я строка заставки
#define text4 "  ver.1.0.8  2020  "                                                 // 4-я строка заставки
  uint16_t  timer;                                                                  // таймер нагрева/охлаждения
  uint16_t  timer2;                                                                 // таймер ухода в Bypass
  uint8_t   symbol1[8] = {00,00,16,16,16,16,00,00};                                 // определяем символ 1 для "одна палка" прямой волны 
  uint8_t   symbol2[8] = {00,00,20,20,20,20,00,00};                                 // определяем символ 2 для "две палки" прямой волны
  uint8_t   symbol3[8] = {00,00,21,21,21,21,00,00};                                 // определяем символ 3 для "три палки" прямой волны

  uint8_t   LT[8] = {07,15,31,31,31,31,31,31};                                      // сегменты цифр
  uint8_t   UB[8] = {31,31,31,00,00,00,00,00};                                      // -------------
  uint8_t   RT[8] = {28,30,31,31,31,31,31,31};                                      // -------------
  uint8_t   LL[8] = {31,31,31,31,31,31,15,07};                                      // -------------
  uint8_t   LB[8] = {00,00,00,00,00,31,31,31};                                      // -------------
  uint8_t   LR[8] = {31,31,31,31,31,31,30,28};                                      // -------------
  uint8_t   UMB[8] ={31,31,31,00,00,00,31,31};                                      // -------------
  uint8_t   LMB[8] ={31,00,00,00,00,31,31,31};                                      // -------------
  uint32_t  start_point;                                                            // счетчик времени градусника
  uint32_t  VoxMillis;                                                              // счетчик времени задержки PTT
  uint32_t  delay_band;                                                             // задержка переключения диапазонов
  int8_t    h, m, s;                                                                // значение часов, минут, секунд таймера обратного отсчета
  uint8_t   new_sec, old_sec, sec, minut, chas;                                     // переменные времени хода часов
  float     SWR=1.0;                                                                // переменная КСВ
  float     Temp;                                                                   // переменная температуры
  uint16_t  PWRMax;                                                                 // 
  uint8_t   PWRStep;                                                                // 
  uint16_t  U_ANOD_MAX;                                                             // 
  uint8_t   U_ANOD_ROUND;                                                           // 
  uint16_t  I_ANOD_MAX;                                                             // 
  uint8_t   I_ANOD_ROUND;                                                           // 
  uint16_t  U_GRID2_MAX;                                                            // 
  uint8_t   U_GRID2_ROUND;                                                          // 
  uint16_t  I_GRID2_MAX;                                                            // 
  uint8_t   I_GRID2_ROUND;                                                          // 
  uint8_t   TIME_ON;                                                                // 
  uint8_t   TIME_OFF;                                                               // 
  uint8_t   TIME_LIGHT;                                                             // 
  uint8_t   TIME_SLEEP;                                                             // 
  uint8_t   STEP;                                                                   // 
  uint8_t   UPDATE_TIME;                                                            // 
  uint8_t   SWR_HI;                                                                 // 
  uint8_t   VOXDelay;                                                               // 
  uint8_t   DELAY_SWITCH;                                                           // 
  
  uint16_t  V, PWR;                                                                 // переменные напряжения, мощности
  uint16_t  Ua;                                                                     // переменная анодного напряжения
  uint16_t  Ugrid2;                                                                 // переменная экранного напряжения
  uint16_t  Ia;                                                                     // переменная тока анода
  uint16_t  Igrid2;                                                                 // переменная тока второй сетки
  uint16_t  Upk;                                                                    // переменная пикового уровня
  uint16_t  valvox;                                                                 // переменная счета отпускания VOX
  bool      flagS=1;                                                                // флаг секунд
  bool      power=0;                                                                // флаг питания
  bool      flag_temp=0;                                                            // флаг индикации температуры
  bool      flag_cool=0;                                                            // флаг режима охлаждения
  bool      flag_swr_hi=0;                                                          // флаг высокого КСВ
  bool      flag_Ua=0;                                                              // флаг наличия анодного напряжения
  uint8_t   band;                                                                   // переменная переключения диапазонов
  uint8_t   band_get;                                                               // переменная выбора диапазонов
  uint16_t  frq, frq_get;                                                           // переменная входной частоты и частоты диапазона
  uint16_t  frq_old1, frq_old2, frq_old3;                                           // переменные промежуточных частот для вычисления частоты диапазона
  word      frqbuf;                                                                 // значение частоты для сохранения в память
  
  uint8_t analog_ref = DEFAULT;                                                     // опорное напряжения +Vcc
  volatile uint16_t ReadUa, ReadIa, ReadUgrid2, ReadIgrid2, ReadUfor, ReadUrev;     // переменные напр. анода, тока анода, напр. второй сетки, тока второй сетки, пр. и обр. волны
  volatile uint8_t analogPin = A0;                                                  // начальный аналоговый пин А0
  volatile bool trueValue = false;                                                  // флаг разрешения прерывания АЦП

  volatile unsigned long timerCounts;                                               // таймер счетчика
  volatile bool counterReady;                                                       // флаг готовности счетчика
  unsigned long overflowCount;                                                      // счетчик переполнений
  unsigned int  timerTicks;                                                         // счетчик тиков
  unsigned int  timerPeriod;                                                        // продолжительность периода измерения
//*********************************************************************************** Подпрограмма частотомера
void startCounting (unsigned int ms){ 
    counterReady= false;                                                            // готовность счётчика
    timerPeriod = ms;                                                               // количество импульсов на входе за 1 мс
    timerTicks  = 0;                                                                // сброс счетчика тиков
    overflowCount = 0;                                                              // сброс счётчика переполнений
    TCCR1A=0; TCCR1B=0; TCCR2A=0; TCCR2B=0;                                         // сброс таймера 1 и таймера 2
    TIMSK1=bit(TOIE1);                                                              // таймер 1-подсчет импульсов на pin D5, прерывание при переполнении таймера 1
    TCCR2A=bit(WGM21) ;                                                             // режим CTC
    OCR2A=124;                                                                      // подсчёт 125 тиков для получения 1мс (корректировка показаний)
    TIMSK2= bit(OCIE2A);                                                            // timer 2-прерывание каждую 1 мс, включаем прерывание Timer2
    TCNT1=0; TCNT2=0;                                                               // сброс счётчиков 1 и 2
    GTCCR=bit(PSRASY);                                                              // сброс предделителя, таймер 2
    TCCR2B=bit(CS20)|bit(CS22);                                                     // предделитель на 128, запуск таймера 1
    TCCR1B=bit(CS10)|bit(CS11)|bit(CS12);}                                          // чтение частоты на T1 пин D5
    ISR (TIMER1_OVF_vect){++overflowCount;}                                         // подсчет числа переполнения счетчика 1, конец TIMER1_OVF_vect                             
    ISR (TIMER2_COMPA_vect){                                                        // таймер 2 прерываний запускается аппаратным таймером 1 каждую 1мс
    unsigned int timer1CounterValue;                                                // получить значение счетчика,перед его изменением
    timer1CounterValue=TCNT1;                                                       // считываем частоту с регистров
    unsigned long overflowCopy=overflowCount;                                       // 
  if (++timerTicks<timerPeriod) return;                                             //
  if ((TIFR1 & bit(TOV1)) && timer1CounterValue < 256) overflowCopy++;              // при переполнении
    TCCR1A=0; TCCR1B=0;                                                             // обнуляем таймер 1
    TCCR2A=0; TCCR2B=0;                                                             // обнуляем таймер 2
    TIMSK1=0; TIMSK2=0;                                                             // отключаем прерывания на Timer1 и Timer2
    timerCounts = (overflowCopy << 16) + timer1CounterValue;                        // подсчёт переполнений, каждое переполнение больше 65536
    counterReady=true;}                                                             // устанавливаем флаг готовности для следующего подсчета

void setup(){//********************************************************************************************
    ADC_init();                                                                     // инициализация аналоговых портов
    beep.setTimeout (50);                                                           // длительность сигнала бипера, мс
    error.setTimeout (750);                                                         // длительность сигнала аварии, мс
    button1.begin();                                                                // инициализация кнопки 1
    lcd.init();                                                                     // производим инициализацию дисплея
    lcd.backlight();                                                                // включаем подсветку
    pinMode(TX,     OUTPUT);                                                        // порт TX на выход
    pinMode(FREQ,   INPUT);                                                         // порт FREQ на вход
    pinMode(SOUND,  OUTPUT);                                                        // порт SOUND на выход
    pinMode(D0,     OUTPUT);                                                        // вывод D0 на выход, подключение дешифратора 8 вывод
    pinMode(D1,     OUTPUT);                                                        // вывод D1 на выход, подключение дешифратора 9 вывод
    pinMode(D2,     OUTPUT);                                                        // вывод D2 на выход, подключение дешифратора 10 вывод
    pinMode(D3,     OUTPUT);                                                        // вывод D3 на выход, подключение дешифратора 11 вывод
    pinMode(FAN,    OUTPUT);                                                        // порт FAN на выход
    pinMode(GLOW,   OUTPUT);                                                        // порт GLOW на выход
    pinMode(SCREEN, OUTPUT);                                                        // порт SCREEN на выход
    pinMode(ANOD,   OUTPUT);                                                        // порт ANOD на выход
    Wire.setClock(800000);                                                          // установка частоты шины I2C 800кГц
//    filter.setCoef(1.0);
//    filter.setStep(1);
    band_get = EEPROM.read(0);                                                      // читаем из памяти информацию о последнем диапазоне перед выключением
    band = band_get;                                                                // приравниваем переменные переключения и выбора диапазонов
  if ((isnan(band_get) == true) || ((band_get < 1) || (band_get > 9))){             // проверяем на нечисло диапазон, рабочий диапазон 1-9
    EEPROM.update(0,5); band_get = EEPROM.read(0); band = band_get;}                // пишем в память 5-й диапазон, переносим его в переменные диапазона    
    
    frqbuf = word (EEPROM.read(1), EEPROM.read(2));                                 // читаем из памяти последнюю частоту
  if ((isnan(frqbuf) == true) || ((frqbuf < 16) || (frqbuf > 300))){                // проверяем на нечисло частоту, рабочая частота 1.7-30.0МГц
    EEPROM.update(1, highByte(141)); EEPROM.update(2, lowByte(141));                // пишем в память частоту 14.1МГц
    frqbuf = word (EEPROM.read(1), EEPROM.read(2));}                                // читаем новое значение в переменную

    PWRMax = word (EEPROM.read(3), EEPROM.read(4));                                 // читаем из памяти значение максимальной мощности
  if ((isnan(PWRMax) == true) || ((PWRMax < 100)|| (PWRMax > 9999))){               // проверяем на нечисло и если мощность меньше 100Вт или больше 9999Вт
    EEPROM.update(3, highByte(1000)); EEPROM.update(4, lowByte(1000));              // пишем в память 1000Вт
    PWRMax = word (EEPROM.read(3), EEPROM.read(4));}                                // читаем новое значение в переменную
    
    PWRStep = EEPROM.read(5);                                                       // читаем из памяти шаг округления показания мощности
  if ((isnan(PWRStep) == true) || ((PWRStep < 1) || (PWRStep > 100))){              // проверяем на нечисло и если шаг мощности меньше 1 или больше 100
    EEPROM.update(5,20); PWRStep = EEPROM.read(5);}                                 // пишем в память 20 и читаем новое значение в переменную

    U_ANOD_MAX = word (EEPROM.read(6), EEPROM.read(7));                             // читаем из памяти максимальное значение анодного напряжения
  if ((isnan(U_ANOD_MAX) == true) || ((U_ANOD_MAX < 100) || (U_ANOD_MAX > 9999))){  // проверяем на нечисло и если значение меньше 100 или больше 9999
    EEPROM.update(6, highByte(3000)); EEPROM.update(7, lowByte(3000));              // пишем в память 3000
    U_ANOD_MAX = word (EEPROM.read(6), EEPROM.read(7));}                            // читаем новое значение в переменную

    U_ANOD_ROUND = EEPROM.read(8);                                                  // читаем из памяти значение шага округления анодного напряжения
  if ((isnan(U_ANOD_ROUND) == true) || ((U_ANOD_ROUND < 1) || (U_ANOD_ROUND > 50))){// проверяем на нечисло и если шаг округления меньше 1 или больше 50
    EEPROM.update(8,20); U_ANOD_ROUND = EEPROM.read(8);}                            // пишем в память 20 и читаем новое значение в переменную

    I_ANOD_MAX = word (EEPROM.read(9), EEPROM.read(10));                            // читаем из памяти значение максимального тока анода в мА
  if ((isnan(I_ANOD_MAX) == true) || ((I_ANOD_MAX < 10) || (I_ANOD_MAX > 990))){    // проверяем на нечисло и если значение меньше 10 или больше 990
    EEPROM.update(9, highByte(990)); EEPROM.update(10, lowByte(990));               // пишем в память 990
    I_ANOD_MAX = word (EEPROM.read(9), EEPROM.read(10));}                           // читаем новое значение в переменную

    I_ANOD_ROUND = EEPROM.read(11);                                                 // читаем из памяти значение шага округления тока анода
  if ((isnan(I_ANOD_ROUND) == true) || ((I_ANOD_ROUND < 1) || (I_ANOD_ROUND > 10))){// проверяем на нечисло и если шаг округления меньше 1 или больше 10
    EEPROM.update(11,10); I_ANOD_ROUND = EEPROM.read(11);}                          // пишем в память 10 и читаем новое значение в переменную

    U_GRID2_MAX = word (EEPROM.read(12), EEPROM.read(13));                          // читаем из памяти максимальное значение анодного напряжения
  if ((isnan(U_GRID2_MAX) == true) || ((U_GRID2_MAX < 100) || (U_GRID2_MAX > 900))){// проверяем на нечисло и если значение меньше 100 или больше 900
    EEPROM.update(12, highByte(650)); EEPROM.update(13, lowByte(650));              // пишем в память 650
    U_GRID2_MAX = word (EEPROM.read(12), EEPROM.read(13));}                         // читаем новое значение в переменную

    U_GRID2_ROUND = EEPROM.read(14);                                                // читаем из памяти значение шага округления напряжения второй сетки
  if ((isnan(U_GRID2_ROUND) == true)||((U_GRID2_ROUND < 1)||(U_GRID2_ROUND > 20))){ // проверяем на нечисло и если шаг округления меньше 1 или больше 20
    EEPROM.update(14,10); U_GRID2_ROUND = EEPROM.read(14);}                         // пишем в память 10 и читаем новое значение в переменную

    I_GRID2_MAX = word (EEPROM.read(15), EEPROM.read(16));                          // читаем из памяти максимальное значение тока второй сетки мА
  if ((isnan(I_GRID2_MAX) == true) || ((I_GRID2_MAX < 5) || (I_GRID2_MAX > 250))){  // проверяем на нечисло и если значение меньше 1 или больше 250
    EEPROM.update(15, highByte(100)); EEPROM.update(16, lowByte(100));              // пишем в память 100
    I_GRID2_MAX = word (EEPROM.read(15), EEPROM.read(16));}                         // читаем новое значение в переменную  
    
    I_GRID2_ROUND = EEPROM.read(17);                                                // читаем из памяти значение шага округления тока второй сетки
  if ((isnan(I_GRID2_ROUND) == true)||((I_GRID2_ROUND < 1)||(I_GRID2_ROUND > 10))){ // проверяем на нечисло и если шаг округления меньше 1 или больше 10
    EEPROM.update(17,5); I_GRID2_ROUND = EEPROM.read(17);}                          // пишем в память 5 и читаем новое значение в переменную
    
    TIME_ON = EEPROM.read(18);                                                      // читаем из памяти значение времени включения (нагрева), сек
  if ((isnan(TIME_ON) == true) || ((TIME_ON < 10) || (TIME_ON > 250))){             // проверяем на нечисло и если время меньше 10 или больше 250 секунд 
    EEPROM.update(18,80); TIME_ON = EEPROM.read(18);}                               // пишем в память 80 и читаем новое значение в переменную

    TIME_OFF = EEPROM.read(19);                                                     // читаем из памяти значение времени отключения (охлаждения), сек
  if ((isnan(TIME_OFF) == true) || ((TIME_OFF < 30) || (TIME_OFF > 250))){          // проверяем на нечисло и если время меньше 30 или больше 250 секунд 
    EEPROM.update(19,120); TIME_OFF = EEPROM.read(19);}                             // пишем в память 120 и читаем новое значение в переменную

    TIME_LIGHT = EEPROM.read(20);                                                   // читаем из памяти значение задержки на отключение подсветки ЖКИ (в минутах)
  if ((isnan(TIME_LIGHT) == true) || ((TIME_LIGHT < 1) || (TIME_LIGHT > 250))){     // проверяем на нечисло и если время меньше 1 или больше 250 минут
    EEPROM.update(20,5); TIME_LIGHT = EEPROM.read(20);}                             // пишем в память 5 и читаем новое значение в переменную
    
    TIME_SLEEP = EEPROM.read(21);                                                   // читаем из памяти значение задержки на переход в спящий режим (в минутах)
  if ((isnan(TIME_SLEEP) == true) || ((TIME_SLEEP < 5) || (TIME_SLEEP > 250))){     // проверяем на нечисло и если время меньше 5 или больше 250 минут
    EEPROM.update(21,20); TIME_SLEEP = EEPROM.read(21);}                            // пишем в память 20 и читаем новое значение в переменную

    STEP = EEPROM.read(22);                                                         // читаем из памяти значение шага уменьшения градусника
  if ((isnan(STEP) == true) || ((STEP < 1) || (STEP > 250))){                       // проверяем на нечисло и если шаг меньше 1 или больше 250
    EEPROM.update(22,10); STEP = EEPROM.read(22);}                                  // пишем в память 10 и читаем новое значение в переменную

    UPDATE_TIME = EEPROM.read(23);                                                  // читаем из памяти значение времени обновления градусника в мс
  if ((isnan(UPDATE_TIME) == true) || ((UPDATE_TIME < 1) || (UPDATE_TIME > 250))){  // проверяем на нечисло и если время меньше 1 или больше 250 мс
    EEPROM.update(23,10); UPDATE_TIME = EEPROM.read(23);}                           // пишем в память 10 и читаем новое значение в переменную

    SWR_HI = EEPROM.read(24);                                                       // читаем из памяти значение КСВ, выше которого блокируется работа усилителя
  if ((isnan(SWR_HI) == true) || ((SWR_HI < 2) || (SWR_HI > 100))){                 // проверяем на нечисло и если КСВ меньше 2 или больше 100
    EEPROM.update(24,5); SWR_HI = EEPROM.read(24);}                                 // пишем в память 5 и читаем новое значение в переменную

    VOXDelay = EEPROM.read(25);                                                     // читаем из памяти значение задержки отпускания VOX
  if ((isnan(VOXDelay) == true) || ((VOXDelay < 1) || (VOXDelay > 100))){           // проверяем на нечисло и если время меньше 1 или больше 100
    EEPROM.update(25,10); VOXDelay = EEPROM.read(25);}                              // пишем в память 10 и читаем новое значение в переменную

    DELAY_SWITCH = EEPROM.read(26);                                                 // читаем из памяти значение задержки на переключение реле диапазонов, мс
  if ((isnan(DELAY_SWITCH) == true)||((DELAY_SWITCH < 1)||(DELAY_SWITCH > 100))){   // проверяем на нечисло и если время меньше 1 или больше 100
    EEPROM.update(26,50); DELAY_SWITCH = EEPROM.read(26);}                          // пишем в память 50 и читаем новое значение в переменную  
    
    lcd.cursor();                                                                   // вкл. курсор
    lcd.setCursor(0,0); lcd.leftToRight();                                          // уст. курсор в (0,0), вывод текста слева направо
    dotimes(strlen(text1),{lcd.write(text1[i]); delay(100);})                       // выводим каждый символ текста text1 с задержкой 100мс
    lcd.setCursor(0,1); lcd.leftToRight();                                          // уст. курсор в (0,1), вывод текста слева направо
    dotimes(strlen(text2),{lcd.write(text2[i]); delay(100);})                       // выводим каждый символ текста text2 с задержкой 100мс
    lcd.setCursor(0,2); lcd.leftToRight();                                          // уст. курсор в (0,2), вывод текста слева направо
    dotimes(strlen(text3),{lcd.write(text3[i]); delay(100);})                       // выводим каждый символ текста text3 с задержкой 100мс
    lcd.setCursor(0,3); lcd.leftToRight();                                          // уст. курсор в (0,2), вывод текста слева направо
    dotimes(strlen(text4),{lcd.write(text4[i]); delay(100);})                       // выводим каждый символ текста text4 с задержкой 100мс
    lcd.noCursor();                                                                 // откл. курсора
    delay(2000);                                                                    // задержка вывода заставки 2сек
    lcd.clear();                                                                    // очистка экрана
    lcd.setCursor(4,1); lcd.print("Standby ...");                                   // выводим "Standby ..."
    lcd.setCursor(0,2); lcd.print("Press S1 for PowerOn");                          // выводим "Press S1 for PowerOn"
    set_band();                                                                     // уст. диапазон
}
void loop(){//*********************************************************************************************** Основной цикл
    global_timer();                                                                 // вызов таймера общего хода времени

  if (beep.isReady())  digitalWrite (SOUND, LOW);                                   // если таймер бипера готов, отключаем бипер
  if (error.isReady()) digitalWrite (SOUND, LOW);                                   // если таймер аварии готов, отключаем сигнал аварии
  
  if (flagS){flagS=0; sensor.requestTemp();                                         // если флаг секунд 1, обнуляем его, запрос температуры
     Temp=sensor.getTemp();}                                                        // получаем температуру на следующей секунде
  
    startCounting(2);    delay(2);                                                  // старт счета частоты, задержка 2мс
    frq = (timerCounts*0.64) / timerPeriod;                                         // получаем входную частоту

  if ((frq>16)&&(!flag_swr_hi)&&(!flag_Ua)){ valvox=0;                              // если частота выше 1.6МГц, не сработала защита по КСВ и анодному напряжению, обнуляем задержку VOX
//  if (power) digitalWrite(TX, HIGH);                                                // если питание включено, переходим на передачу
    timer2=0; lcd.backlight();                                                      // сбрасываем второй таймер, включаем подсветку индикатора
  for (byte i=0; i<3; i++){startCounting(2); delay(2);                              // вкл. первый цикл, старт счета, задержка на подсчет 2мс
    frq = (timerCounts*0.64) / timerPeriod;                                         // получаем входную частоту
  if (frq>frq_old1) frq_old1=frq;}                                                  // если частота frq больше промежуточной frq_old1, перезаписываем в frq_old1 значение frq
  for (byte i=0; i<3; i++){startCounting(2); delay(2);                              // вкл. второй цикл, старт счета, задержка на подсчет 2мс
    frq = (timerCounts*0.64) / timerPeriod;                                         // получаем входную частоту
  if (frq>frq_old2) frq_old2=frq;}                                                  // если частота frq больше промежуточной frq_old2, перезаписываем в frq_old2 значение frq
  for (byte i=0; i<3; i++){startCounting(2); delay(2);                              // вкл. третий цикл, старт счета, задержка на подсчет 2мс
    frq = (timerCounts*0.64) / timerPeriod;                                         // получаем входную частоту
  if (frq>frq_old3) frq_old3=frq;}                                                  // если частота frq больше промежуточной frq_old3, перезаписываем в frq_old3 значение frq
  if ((frq_old1==frq_old2) && (frq_old2==frq_old3)) frq_get=frq_old1;               // проверяем равенство частот frq_old1=frq_old2=frq_old3
  switch (frq_get){                                                                 // 
    case 17  ...  30: band_get=1; break;                                            // получаем диапазон для переключения 160м(1)
    case 31  ...  50: band_get=2; break;                                            // получаем диапазон для переключения 80м (2)
    case 51  ...  80: band_get=3; break;                                            // получаем диапазон для переключения 40м (3)
    case 81  ... 120: band_get=4; break;                                            // получаем диапазон для переключения 30м (4)
    case 121 ... 160: band_get=5; break;                                            // получаем диапазон для переключения 20м (5)
    case 161 ... 190: band_get=6; break;                                            // получаем диапазон для переключения 17м (6)
    case 191 ... 230: band_get=7; break;                                            // получаем диапазон для переключения 15м (7)
    case 231 ... 260: band_get=8; break;                                            // получаем диапазон для переключения 12м (8)
    case 261 ... 300: band_get=9; break;}                                           // получаем диапазон для переключения 10м (9)
  if (band_get != band){ beep.start(); digitalWrite(SOUND, HIGH);                   // если текущий диапазон не равен новому значению, бипер,
    delay_band=millis();                                                            // принимаем текущее значение времени
    band = band_get;  EEPROM.update(0, band_get);                                   // переписываем новое значение в текущий, обновляем в памяти
    frqbuf = frq_get; EEPROM.update(1, highByte(frqbuf));                           // переписываем новое значение частоты, обновляем в памяти
    EEPROM.update(2, lowByte(frqbuf)); set_band();}}                                // и устанавливаем новый диапазон

  else if (millis() - VoxMillis >= 1){VoxMillis = millis(); valvox++;}              // формируем задержку на выключение, пока счетчик valvox считает
  else valvox = 0;                                                                  // иначе обнуляем valvox
  if (valvox >= VOXDelay){valvox = 0; digitalWrite(TX, LOW);}                       // как только досчитали, переходим на прием
  if (frq < frq_old1){frq_old1 = 0; frq_old2 = 0; frq_old3 = 0; frq_get=0;}         // если новая частота меньше предыдущей обнуляем frq_old...
  
  if ((power)&&(frq_get>16)&&(millis()-delay_band>DELAY_SWITCH))                    // если питание подано, частота выше 1.6МГц и задержка больше значения DELAY_SWITCH
    digitalWrite(TX, HIGH);                                                         // переходим на передачу
  
  switch(button1.Loop()){                                                           // кнопка 1
  case SB_CLICK: beep.start(); digitalWrite(SOUND, HIGH); timer2=0; lcd.backlight();// короткое нажатие: бипер 50мс, сбрасываем timer2, включаем подсветку
  if ((flag_swr_hi)&&(!flag_Ua)){flag_swr_hi=0; digitalWrite(SCREEN, HIGH);         // если флаг высокого КСВ равен 1, сбрасываем его в 0, включаем экр. напряжение
    delay(100); break;}                                                             // задержка 100мс, выход из кнопки 1
  
  if (power){flag_temp=!flag_temp;                                                  // если флаг питания 1, меняем флаг температуры
    lcd.setCursor(0,3); lcd.print("                    ");                          // стираем нижнюю строку
  if (flag_temp)  {lcd.setCursor(0,3); lcd.print("T=            ");}                // если флаг температуры 1, в нижней строке выводим "T=            "
  if (!flag_temp) {lcd.setCursor(0,3); lcd.print("Ug=        Ig=");}}               // если флаг температуры 0, в нижней строке выводим "Ug=        Ig="
  break;                                                                            // конец функции короткого нажатия
  
  case SB_LONG_CLICK: beep.start(); digitalWrite(SOUND, HIGH); power=!power;        // длинное нажатие: бипер, переворачиваем флаг питания
    flag_cool=!flag_cool;                                                           // взводим таймер счета
    flag_swr_hi=0; flag_Ua=0; sec=0; minut=0; timer2=0; lcd.clear(); CharClock();   // обнуляем флаг высокого КСВ, обнуляем секунды и минуты, загружаем символы в индикатор
    lcd.setCursor(9,1); lcd.write(165); lcd.setCursor(9,2); lcd.write(165);         // выводим двоеточие между цифрами
  if (power)  heating();                                                            // если флаг питания 1, уходим на нагрев
  if (!power) cooling();                                                            // если флаг питания 0, уходим на охлаждение
    break;}                                                                         // конец функции кнопки 1
//************************************************************************************ Управление подсветкой и переход в Bypass по времени  
  if (ReadUfor>10) {timer2=0; lcd.backlight();}                                     // если есть выходной сигнал, обнуляем таймер2, включаем подсветку ЖКИ
  else {                                                                            // если сигнал отсутствует
  if (TIME_LIGHT==timer2) lcd.noBacklight();                                        // при достижении времени до TIME_LIGHT отключаем подсветку
  if ((TIME_SLEEP==timer2)&&(flag_cool)){power=!power; sec=0; minut=0; lcd.clear(); // при достижении времени до TIME_SLEEP переворачиваем флаг питания, обнуляем секунды и минуты, стираем ЖКИ
    CharClock(); lcd.setCursor(9,1); lcd.write(165); lcd.setCursor(9,2);            // загружаем сегменты цифр в ОЗУ индикатора, выводим двоеточие
    lcd.write(165); cooling(); flag_cool=0;}}                                       // уходим на охлаждение, обнуляем флаг охлаждения
  if (power){                                                                       // если флаг питания включен
//************************************************************************************ Градусник прямой волны  
  if (ReadUfor>Upk) Upk=ReadUfor;                                                   // если уровень с порта больше пикового значения, выравниваем показания
  if (Upk && (millis()-start_point>UPDATE_TIME)){                                   // если пиковое значение и время больше 0, но меньше UPDATE_TIME
    start_point=millis();                                                           // выравниваем время с start_point
  if (Upk<STEP) Upk=0;                                                              // проверяем уровень Upk, если он меньше шага STEP, то обнуляем
  else Upk-=STEP;}                                                                  // если больше шага STEP, то на каждый цикл уменьшаем Upk на шаг STEP
    byte a=Upk/63;                                                                  // вычисляем целое число a для градусника
    byte b=Upk-a*63;                                                                // вычисляем значение переменной b остатка от деления
      lcd.setCursor(0,0);                                                           // уст. курсор начала градусника
  for (byte i=0; i<a; i++) lcd.write(3);                                            // выводим целую часть, если она больше 0
  switch(b){                                                                        // выводим дробную часть градусника
    case 0:         lcd.write(32); break;                                           // если b=0
    case 1  ... 21: lcd.write(1);  break;                                           // если b=1...17
    case 22 ... 42: lcd.write(2);  break;                                           // если b=18...34
    case 43 ... 63: lcd.write(3);  break;}                                          // если b=35...51
  if (a<15) lcd.write(32);                                                          // в конце пробел, если a<15
//************************************************************************************ Расчет КСВ
  if ((ReadUfor==0)&&(ReadUrev==0)){SWR=1.0;}                                       // в отсутствии сигнала КСВ=1
  if ((ReadUfor!=0)&&(ReadUrev!=0)&&(ReadUrev>=ReadUfor)){SWR=9.9;}                 // если напряжение пр. и обр. волны равны или обр. больше прямой, то КСВ максимальное
  if (ReadUfor>ReadUrev){SWR=(float)(ReadUfor+ReadUrev)/(float)(ReadUfor-ReadUrev);}// если ReadUfor>ReadUrev, то выводим значение КСВ, расчитанное по формуле
  if (SWR>=10){SWR=9.9;}                                                            // ограничиваем КСВ до значения 9.9
  if (SWR>3.0) {error.start(); digitalWrite(SOUND, HIGH);}                          // при КСВ выше 3.0 подаем сигнал частотой toneSWR на порт SoundPin
  if (SWR>SWR_HI){flag_swr_hi=1; digitalWrite(SCREEN,LOW);}                         // если КСВ больше SWR_HI, то флаг высокого КСВ уст. в 1, откл. экран. напряжение
     lcd.setCursor(15,1);                                                           // курсор в позицию 15,1
  if (flag_swr_hi) lcd.print("HIGH ");                                              // если флаг высокого КСВ равен 1, выводим "HI"
  else  {lcd.print(SWR,1); lcd.print(" ");}                                         // если равен 0, то выводим значение КСВ до десятых
//************************************************************************************ Расчёт мощности
      PWR=Upk;                                                                      // 
uint16_t PWR_rnd=PWR/PWRStep; PWR=PWR_rnd*PWRStep;                                  // округляем мощность кратно PWRStep
      lcd.setCursor(4,1); lcd.print(PWR); lcd.print("W  ");                         // выводим значение мощности
//************************************************************************************ Расчёт и вывод напряжения анода
      Ua=map(ReadUa,0,1023,0,U_ANOD_MAX);                                           // масштабируем показания напряжения анода
//ReadUa=filter.filteredTime(ReadUa);                                               // попытка отфильтровать показания
uint16_t Ua_rnd=Ua/U_ANOD_ROUND; Ua=Ua_rnd*U_ANOD_ROUND;                            // округляем показания напряжения анода
  if ((Ua<(2*Ugrid2))&&(!flag_Ua)){flag_Ua=1;                                       // если напр. анода ниже удвоенного экранного и флаг пропадания анодн. напряжения 0, флаг уст. в 1
      digitalWrite(SCREEN,LOW); digitalWrite(ANOD,LOW);                             // отключаем экранное и анодное напряжения
      error.start(); digitalWrite(SOUND, HIGH);}                                    // сигнал аварии
      lcd.setCursor(3,2);                                                           // курсор в позицию 3,2
  if (flag_Ua) {lcd.setCursor(3,2); lcd.print("ERROR ");}                           // если сработал флаг пропадания анодного напряжения, выводим ERROR
  else {lcd.print(Ua); lcd.print("V   ");}                                          // иначе выводим на экран значение напряжения, добавляем в конце "V"
//************************************************************************************ Расчёт и вывод напряжения второй сетки
      Ugrid2=map(ReadUgrid2,0,1023,0,U_GRID2_MAX);                                  // масштабируем показания напряжения второй сетки
uint16_t Ugrid2_rnd=Ugrid2/U_GRID2_ROUND;                                           // округляем показания напряжения анода
         Ugrid2=Ugrid2_rnd*U_GRID2_ROUND;                                           // получаем новое значение
      lcd.setCursor(3,3);                                                           // курсор в позицию 3,3
  if (!flag_temp) {                                                                 // если флаг температуры равен 0
  if ((flag_swr_hi)||(flag_Ua)) lcd.print("OFF   ");                                // если флаг высокого КСВ равен 1, выводим "OFF"
  else {lcd.print(Ugrid2); lcd.print("V   ");}}                                     // иначе выводим на экран напряжение, добавляем в конце "V"
  if (flag_temp) {lcd.setCursor(2,3); lcd.print(Temp,1);                            // если флаг температуры равен 1, выводим на экран значение температуры с позиции 2,3
  lcd.print(char(223)); lcd.print("C ");}                                           // добавляем символ градуса и добавляем "C "

//************************************************************************************ Расчёт и вывод тока анода
      Ia=map(ReadIa,0,246,0,I_ANOD_MAX);                                            // масштабируем показания тока анода
    if (I_ANOD_ROUND>1){uint16_t Ia_rnd=Ia/I_ANOD_ROUND;                            // округляем показания тока анода, если шаг округления больше 1
      Ia=Ia_rnd*I_ANOD_ROUND;}                                                      // ------------------------
      lcd.setCursor(14,2);                                                          // выводим значение тока анода на экран
    if (Ia<100) {lcd.print(Ia); lcd.print("mA  ");}                                 // если показания меньше 100
    else {lcd.print(Ia); lcd.print("mA ");}                                         // если больше 100
//************************************************************************************ Расчёт и вывод тока второй сетки
      Igrid2=map(ReadIgrid2,0,61,0,I_GRID2_MAX);                                    // масштабируем показания тока второй сетки
                                                                                    // выводим значение тока второй сетки на экран
    if (flag_temp==0) {lcd.setCursor(14,3);                                         // если флаг температуры равен 0, в позиции курсора 14,3
    if (Igrid2<100) {lcd.print(Igrid2); lcd.print("mA  ");}                         // выводим ток второй сетки, если показания меньше 100
    else {lcd.print(Igrid2); lcd.print("mA ");}}                                    // выводим ток второй сетки, если показания больше 100
    
    else {lcd.setCursor(11,3);                                                      // если флаг температуры равен 1, в позиции курсора 11,3
    if (chas<10){lcd.print("0"); lcd.print(chas);}                                  // выводим часы, если значение меньше 10
    else lcd.print(chas);                                                           // если значение больше 10
      lcd.print(":");                                                               // добавляем двоеточие
    if (minut<10){lcd.print("0"); lcd.print(minut);}                                // выводим минуты, если значение меньше 10
    else lcd.print(minut);                                                          // если значение больше 10
      lcd.print(":");                                                               // добавляем двоеточие
    if (sec<10){lcd.print("0"); lcd.print(sec);}                                    // выводим секунды, если значение меньше 10
    else lcd.print(sec); lcd.print(" ");}                                           // если значение больше 10
}}
//************************************************************************************ Подпрограмма сегментов градусника
void CharBar() {                                                                    // заносим сегменты градусника в память ОЗУ индикатора
    lcd.createChar(1,symbol1);                                                      // загружаем символ из массива symbol1 в 1 ячейку ОЗУ дисплея
    lcd.createChar(2,symbol2);                                                      // загружаем символ из массива symbol2 во 2 ячейку ОЗУ дисплея 
    lcd.createChar(3,symbol3);                                                      // загружаем символ из массива symbol3 в 3 ячейку ОЗУ дисплея
}
//************************************************************************************ Подпрограмма сегментов цифр
void CharClock() {                                                                  // заносим сегменты в память ОЗУ индикатора
    lcd.createChar(0, LT);                                                          // -------------------------
    lcd.createChar(1, UB);                                                          // -------------------------
    lcd.createChar(2, RT);                                                          // -------------------------
    lcd.createChar(3, LL);                                                          // -------------------------
    lcd.createChar(4, LB);                                                          // -------------------------
    lcd.createChar(5, LR);                                                          // -------------------------
    lcd.createChar(6, UMB);                                                         // -------------------------
    lcd.createChar(7, LMB);}                                                        // -------------------------
//************************************************************************************ Подпрограмма вывода минут
void drawMin(byte minutes, byte x, byte y) {                                        // 
    drawDig(minutes/10,x,y);                                                        // 
    drawDig(minutes%10,x+3,y);}                                                     // 
//************************************************************************************ Подпрограмма вывода часов
void drawSec(byte seconds, byte x, byte y) {                                        // 
    drawDig(seconds/10,x+7,y);                                                      // 
    drawDig(seconds%10,x+10,y);}                                                    // 
//************************************************************************************ Подпрограмма вывода сегментов цифр
void drawDig(byte dig, byte x, byte y) {                                            // 
  switch (dig) {                                                                    // 
    case 0:                                                                         // цифра "0"
      lcd.setCursor(x, y); lcd.write(0); lcd.write(1); lcd.write(2);                // 
      lcd.setCursor(x, y + 1); lcd.write(3); lcd.write(4); lcd.write(5); break;     // 
    case 1:                                                                         // цифра "1"
      lcd.setCursor(x, y); lcd.write(32); lcd.write(1); lcd.write(2);               // 
      lcd.setCursor(x, y + 1); lcd.write(32); lcd.write(32); lcd.write(5); break;   // 
    case 2:                                                                         // цифра "2"
      lcd.setCursor(x, y); lcd.write(6); lcd.write(6); lcd.write(2);                // 
      lcd.setCursor(x, y + 1); lcd.write(3); lcd.write(7); lcd.write(7); break;     // 
    case 3:                                                                         // цифра "3"
      lcd.setCursor(x, y); lcd.write(6);lcd.write(6);lcd.write(2);                  // 
      lcd.setCursor(x, y + 1); lcd.write(7); lcd.write(7); lcd.write(5); break;     // 
    case 4:                                                                         // цифра "4"
      lcd.setCursor(x, y); lcd.write(3); lcd.write(4); lcd.write(2);                // 
      lcd.setCursor(x, y + 1); lcd.write(32); lcd.write(32);lcd.write(5); break;    // 
    case 5:                                                                         // цифра "5"
      lcd.setCursor(x, y); lcd.write(0); lcd.write(6); lcd.write(6);                // 
      lcd.setCursor(x, y + 1); lcd.write(7); lcd.write(7); lcd.write(5); break;     // 
    case 6:                                                                         // цифра "6"
      lcd.setCursor(x, y); lcd.write(0); lcd.write(6); lcd.write(6);                // 
      lcd.setCursor(x, y + 1); lcd.write(3); lcd.write(7); lcd.write(5); break;     // 
    case 7:                                                                         // цифра "7"
      lcd.setCursor(x, y); lcd.write(1); lcd.write(1); lcd.write(2);                // 
      lcd.setCursor(x, y + 1); lcd.write(32); lcd.write(0); lcd.write(32); break;   // 
    case 8:                                                                         // цифра "8"
      lcd.setCursor(x, y); lcd.write(0); lcd.write(6); lcd.write(2);                // 
      lcd.setCursor(x, y + 1); lcd.write(3); lcd.write(7); lcd.write(5); break;     // 
    case 9:                                                                         // цифра "9"
      lcd.setCursor(x, y); lcd.write(0); lcd.write(6); lcd.write(2);                // 
      lcd.setCursor(x, y + 1); lcd.write(7); lcd.write(7); lcd.write(5); break;}}   // 
//************************************************************************************ Настройка АЦП
void ADC_init(){                                                                    // Подпрограмма настройки АЦП
  ADCSRA = 0;                                                                       // Сбрасываем регистр ADCSRA
  ADCSRB = 0;                                                                       // Сбрасываем регистр ADCSRB
  ADMUX |= (1 << REFS0);                                                            // Задаем ИОН
  analog_ref = ADMUX;                                                               // Запоминаем состояние регистра - из него мы будем формировать маску для смены входного пина
  ADMUX |= (0 & 0x07);                                                              // Выбираем начальный пин A0 для преобразования
                                                                                    // Устанавливаем предделитель - [111-128] [110-64] [101-32] [100-16] [011-8] 
  ADCSRA |= (1 << ADPS2) | (1 << ADPS0);                                            // Биты ADPS2 и ADPS0
  ADCSRA &= ~ (0 << ADPS1);                                                         // Бит ADPS1
  ADCSRA |= (1 << ADATE);                                                           // Включаем автоматическое преобразование
  ADCSRA |= (1 << ADIE);                                                            // Разрешаем прерывания по завершении преобразования
  ADCSRA |= (1 << ADEN);                                                            // Включаем АЦП
  ADCSRA |= (1 << ADSC);                                                            // Запускаем преобразование
}
//************************************************************************************ Работа АЦП по прерыванию
ISR(ADC_vect) {                                                                     // 
  if (trueValue) {                                                                  // если флаг смены входного пина АЦП
    int result = ADCL | (ADCH << 8);                                                // Получаем 10-битный результат
  if (analogPin == A0) ReadUa=result;                                               // Если входной пин A0, получаем значение анодного напряжения
//    ReadUa=filter.filteredTime(result);
  if (analogPin == A1) {ReadIa = result;     if (ReadIa>=205)    ReadIa=205;}       // Если входной пин A1, получаем значение тока анода, ограничиваем до значения 205 (1200мВ)
  if (analogPin == A2) ReadUgrid2  = result;                                        // Если входной пин A2, получаем значение напряжения второй сетки
  if (analogPin == A3) {ReadIgrid2 = result; if (ReadIgrid2>=61) ReadIgrid2=61;}    // Если входной пин A3, получаем значение тока второй сетки
  if (analogPin == A6) {ReadUfor   = result; if (ReadUfor>=1008) ReadUfor=1008;}    // Если входной пин A6, получаем значение напряжения прямой волны, ограничение до 1008
  if (analogPin == A7) {ReadUrev   = result; if (ReadUrev>=1008) ReadUrev=1008;}    // Если входной пин A7, получаем значение напряжения обратной волны, ограничение до 1008
      analogPin++;                                                                  // Перебираем входные пины по кругу (А0...А7)
  if (analogPin == A4) analogPin = A6;                                              // Пропускаем А4 и А5
  if (analogPin >  A7) analogPin = A0;                                              // После А7 переходим к А0
    int pin = analogPin;                                                            // Приводим пин вида A0 (14) к удобному для регистра ADMUX - 0. A0=14, A1=15, ...
  if (pin >= 14) pin -= 14;                                                         // Переходим к пину А0
    ADMUX = analog_ref | (pin & 0x07);                                              // Устанавливаем новый вход для преобразования
    trueValue = false;}                                                             // Устанавливаем флаг смены входного пина - следующее прерывание пропускаем
  else trueValue = true;                                                            // Первый раз пропускаем считывание и устанавливаем флаг на чтение в следующий раз
}
//************************************************************************************ Старт индикатора
void startLCD(){
    lcd.clear();                                                                    // очищаем дисплей
    CharBar();                                                                      // загружаем сегменты градусника в ОЗУ индикатора
    flag_temp=0;                                                                    // режим измерения напряжения и тока второй сетки
    lcd.setCursor(0,1);   lcd.print("PWR=");                                        // выводим на индикатор информацию
    lcd.setCursor(11,1);  lcd.print("SWR=");                                        // -------------------------------
    lcd.setCursor(0,2);   lcd.print("Ua=");                                         // -------------------------------
    lcd.setCursor(11,2);  lcd.print("Ia=");                                         // -------------------------------
    lcd.setCursor(0,3);   lcd.print("Ug=");                                         // -------------------------------
    lcd.setCursor(11,3);  lcd.print("Ig=");                                         // -------------------------------
}
//************************************************************************************ Подпрограмма режима нагрева
void heating(){
    timer=TIME_ON; m=timer/60; s=timer-m*60; lcd.backlight();                       // взводим таймер timer, вычисляем минуты и секунды, включаем подсветку
    lcd.setCursor(4,0); lcd.print("HEATING ...");                                   // выводим "HEATING ..."
  while (timer!=0){global_timer(); drawSec(s,3,1); drawMin(m,3,1);                  // цикл включения, пока счетчик timer не равен нулю
  if ((TIME_ON-timer)==2) {digitalWrite(FAN,   HIGH);                               // на второй секунде счета включаем вентилятор охлаждения
    lcd.setCursor(4,3); lcd.print("Start  FAN");}                                   // выводим "Start FAN"
  if ((TIME_ON-timer)==4) {digitalWrite(GLOW,  HIGH);                               // на четвертой секунде счета включаем накал
    lcd.setCursor(4,3); lcd.print("Start  GLOW");}                                  // выводим "Start GLOW"
  if ((TIME_ON-timer)==TIME_ON-4) {digitalWrite(ANOD,    HIGH);                     // за четыре секунды до конца счета включаем анодное напряжение
    lcd.setCursor(4,3); lcd.print("Start  ANOD");}                                  // выводим "Start ANOD"
  if ((TIME_ON-timer)==TIME_ON-2) {                                                 // за две секунды до конца счета включаем экранное напряжение
    Ua=map(ReadUa,0,1023,0,U_ANOD_MAX);                                             // получаем анодное напряжение
  if (Ua<2*U_GRID2_MAX){digitalWrite(ANOD, LOW); flag_Ua=1;                         // если анодное напряжение ниже удвоенного экранного, откл. анодное напряжение, уст. флаг в 1
    lcd.setCursor(4,3); lcd.print("ANOD  ERROR"); delay(500); break;}               // выводим "ANOD  ERROR" и прерываем цикл нагрева
  else {                                                                            // если напряжение в норме
    digitalWrite(SCREEN,  HIGH);                                                    // подаем экранное напряжение
    lcd.setCursor(4,3); lcd.print("Start SCREEN");}}                                // выводим "Start SCREEN"
  if (beep.isReady())  digitalWrite (SOUND, LOW);}                                  // 
  if (flag_Ua) {error.start(); digitalWrite(SOUND, HIGH);}                          // если сработал флаг исчезновения анодного напряжения, даем сигнал аварии
  else {beep.start(); digitalWrite(SOUND, HIGH);}                                   // если флаг исчезновения анодного напряжения не сработал, то даем длинный сигнал готовности
    startLCD(); set_band();}                                                        // запускаем рабочий экран индикатора

//************************************************************************************ Подпрограмма режима охлаждения
void cooling(){
    timer=TIME_OFF; m=timer/60; s=timer-m*60;                                       // взводим таймер timer, вычисляем минуты и секунды
    lcd.setCursor(4,0); lcd.print("COOLING ...");                                   // выводим "COOLING ..."
  while (timer!=0){global_timer(); drawSec(s,3,1); drawMin(m,3,1);                  // цикл отключения, пока счетчик timer не равен нулю
  if ((TIME_OFF-timer)==2) {digitalWrite(SCREEN, LOW);                              // на второй секунде отключаем экранное напряжение
    lcd.setCursor(4,3); lcd.print("Stop SCREEN");}                                  // выводим "Stop SCREEN"
  if ((TIME_OFF-timer)==4) {digitalWrite(ANOD,   LOW);                              // на четвертой секунде отключаем анодное напряжение
    lcd.setCursor(4,3); lcd.print("Stop  ANOD ");}                                  // выводим "Stop  ANOD "
  if ((TIME_OFF-timer)==6) {digitalWrite(GLOW,   LOW);                              // на шестой секунде отключаем накал
    lcd.setCursor(4,3); lcd.print("Stop  GLOW");}                                   // выводим "Stop  GLOW"
  if ((TIME_OFF-timer)==TIME_OFF-2) {digitalWrite(FAN, LOW);                        // за две секунды до конца счета отключаем вентилятор охлаждения
    lcd.setCursor(4,3); lcd.print("Stop  FAN ");}                                   // выводим "Stop  FAN "
  if (beep.isReady())  digitalWrite (SOUND, LOW);}                                  // если таймер beep закончил счет, отключаем бипер
    beep.start(); digitalWrite(SOUND, HIGH); lcd.clear();                           // бипер, очищаем экран
    lcd.setCursor(4,1); lcd.print("Standby ...");                                   // выводим "Standby ..." во второй строке
    lcd.setCursor(0,2); lcd.print("Press S1 for PowerOn");                          // выводим "Press S1 for PowerOn" в третьей строке
    set_band(); lcd.noBacklight();}                                                 // отключаем подсветку
//************************************************************************************ Общий таймер хода часов
void global_timer(){                                                                // 
    new_sec =(millis()/1000)%60;                                                    // таймер 1 секунда
  if (new_sec!=old_sec){old_sec=new_sec; flagS=1;                                   // с каждой новой секундой флаг секунд в 1
  if (++sec>59) {sec=0; minut++; timer2++;}                                         // считаем секунды, при переполнении увел. минуты и таймер 2
  if (minut>59) {minut=0; chas++;}                                                  // при переполнении минут увел. часы
  if (chas>23)  {chas=0;}                                                           // при переполнении часов сбрасываем их в 0
  if (timer>0)  {timer--; s--;                                                      // если таймер обратного хода больше 0, уменьшаем секунды
  if (s<0) {s=59; m--;}}}                                                           // при обратном переполнении секунд уст. на 59, уменьшаем минуты
}
//************************************************************************************ Выбор диапазона
void set_band(){
  if (!power){                                                                      // если питание отключено (байпас)
    lcd.setCursor(2,0); lcd.print("FRQ=");                                          // выводим надпись "FRQ="
    lcd.print((float)frqbuf/10,1); lcd.print("MHz  ");                              // и значение частоты формата ХХ.ХМГц
  switch (band){                                                                    // переключаем диапазоны и выводим
    case 1: lcd.print("160m");                                                      // ----------------- 160m -------------------
      digitalWrite(D0, HIGH); digitalWrite(D1, LOW);                                // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, LOW);  digitalWrite(D3, LOW);  break;                        // --------- D3=0--D2=0--D1=0--D0=1 ---------
    case 2: lcd.print(" 80m");                                                      // ------------------ 80m -------------------
      digitalWrite(D0, LOW); digitalWrite(D1, HIGH);                                // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, LOW); digitalWrite(D3, LOW);  break;                         // --------- D3=0--D2=0--D1=1--D0=0 ---------
    case 3: lcd.print(" 40m");                                                      // ------------------ 40m -------------------
      digitalWrite(D0, HIGH); digitalWrite(D1,HIGH);                                // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, LOW);  digitalWrite(D3,LOW);  break;                         // --------- D3=0--D2=0--D1=1--D0=1 ---------
    case 4: lcd.print("30m");                                                       // ------------------ 30m -------------------
      digitalWrite(D0, LOW);  digitalWrite(D1, LOW);                                // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, HIGH); digitalWrite(D3, LOW); break;                         // --------- D3=0--D2=1--D1=0--D0=0 ---------
    case 5: lcd.print("20m");                                                       // ------------------ 20m -------------------
      digitalWrite(D0, HIGH); digitalWrite(D1, LOW);                                // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, HIGH); digitalWrite(D3, LOW); break;                         // --------- D3=0--D2=1--D1=0--D0=1 ---------
    case 6: lcd.print("17m");                                                       // ------------------ 17m -------------------
      digitalWrite(D0, LOW);  digitalWrite(D1, HIGH);                               // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, HIGH); digitalWrite(D3, LOW); break;                         // --------- D3=0--D2=1--D1=1--D0=0 --------- 
    case 7: lcd.print("15m");                                                       // ------------------ 15m -------------------
      digitalWrite(D0, HIGH); digitalWrite(D1, HIGH);                               // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, HIGH); digitalWrite(D3, LOW); break;                         // --------- D3=0--D2=1--D1=1--D0=1 --------- 
    case 8: lcd.print("12m");                                                       // ------------------ 12m -------------------
      digitalWrite(D0, LOW); digitalWrite(D1, LOW);                                 // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, LOW); digitalWrite(D3, HIGH); break;                         // --------- D3=1--D2=0--D1=0--D0=0 --------- 
    case 9: lcd.print("10m");                                                       // ------------------ 10m -------------------
      digitalWrite(D0, HIGH); digitalWrite(D1, LOW);                                // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, LOW);  digitalWrite(D3, HIGH); break;}}                      // --------- D3=1--D2=0--D1=0--D0=1 ---------
  
  if (power){                                                                       // если питание подано
    lcd.setCursor(16,0);                                                            // курсор в позицию 16,0
  switch (band){                                                                    // выводим диапазон
    case 1: lcd.print("160m");                                                      // ----------------- 160m -------------------
      digitalWrite(D0, HIGH); digitalWrite(D1, LOW);                                // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, LOW);  digitalWrite(D3, LOW);  break;                        // --------- D3=0--D2=0--D1=0--D0=1 ---------
    case 2: lcd.print(" 80m");                                                      // ------------------ 80m -------------------
      digitalWrite(D0, LOW); digitalWrite(D1, HIGH);                                // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, LOW); digitalWrite(D3, LOW);   break;                        // --------- D3=0--D2=0--D1=1--D0=0 ---------
    case 3: lcd.print(" 40m");                                                      // ------------------ 40m -------------------
      digitalWrite(D0, HIGH); digitalWrite(D1,HIGH);                                // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, LOW);  digitalWrite(D3,LOW);   break;                        // --------- D3=0--D2=0--D1=1--D0=1 ---------
    case 4: lcd.print(" 30m");                                                      // ------------------ 30m -------------------
      digitalWrite(D0, LOW);  digitalWrite(D1, LOW);                                // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, HIGH); digitalWrite(D3, LOW);  break;                        // --------- D3=0--D2=1--D1=0--D0=0 ---------
    case 5: lcd.print(" 20m");                                                      // ------------------ 20m -------------------
      digitalWrite(D0, HIGH); digitalWrite(D1, LOW);                                // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, HIGH); digitalWrite(D3, LOW);  break;                        // --------- D3=0--D2=1--D1=0--D0=1 ---------
    case 6: lcd.print(" 17m");                                                      // ------------------ 17m -------------------
      digitalWrite(D0, LOW);  digitalWrite(D1, HIGH);                               // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, HIGH); digitalWrite(D3, LOW);  break;                        // --------- D3=0--D2=1--D1=1--D0=0 --------- 
    case 7: lcd.print(" 15m");                                                      // ------------------ 15m -------------------
      digitalWrite(D0, HIGH); digitalWrite(D1, HIGH);                               // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, HIGH); digitalWrite(D3, LOW);  break;                        // --------- D3=0--D2=1--D1=1--D0=1 --------- 
    case 8: lcd.print(" 12m");                                                      // ------------------ 12m -------------------
      digitalWrite(D0, LOW); digitalWrite(D1, LOW);                                 // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, LOW); digitalWrite(D3, HIGH);  break;                        // --------- D3=1--D2=0--D1=0--D0=0 --------- 
    case 9: lcd.print(" 10m");                                                      // ------------------ 10m -------------------
      digitalWrite(D0, HIGH); digitalWrite(D1, LOW);                                // вывод кода для дешифратора диапазонных ФНЧ
      digitalWrite(D2, LOW);  digitalWrite(D3, HIGH); break;}}                      // --------- D3=1--D2=0--D1=0--D0=1 ---------
}
